# Copyright (C) 2008, OLPC
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA


from gettext import gettext as _
import logging

import gconf
import gobject
import gtk
import pango

from sugar.graphics import style
from sugar.graphics.alert import Alert
from sugar.graphics.icon import Icon

from jarabe.controlpanel.sectionview import SectionView
from jarabe.controlpanel.inlinealert import InlineAlert


CLASS = 'Network'
ICON = 'module-network'
TITLE = _('Network')

_APPLY_TIMEOUT = 3000
EXPLICIT_REBOOT_MESSAGE = _('Please restart your computer for changes to take effect.')


class WrappedLabel(gtk.Label):
    """Variant of gtk.Label that automatically line-wraps the text as needed.

    In theory, stock gtk.Label can already do this, but in practice a
    long-standing bug in GTK prevents it from working as expected.
    This class includes a workaround for that bug.

    Cave-at: WrappedLabel.set_alignment() will stop working.
    """

    def __init__(self, string=None):
        gtk.Label.__init__(self, str=string)
        self.set_line_wrap(True)
        self.connect('size-allocate', self.__size_allocate_cb)

    def __size_allocate_cb(self, widget, rect):
        """Propagate parent size allocation to gtk.Label"""
        # This is a workaround for GTK bug #318276
        # https://bugzilla.gnome.org/show_bug.cgi?id=318276
        widget.set_size_request(rect.width, -1)


class GConfMixin(object):
    """Mix-in class for GTK widgets backed by GConf

    It is the callers responsibility to call GConfClient.add_dir() for the
    GConf directory containing the key.
    """

    def __init__(self, gconf_key, widget=None, signal='changed'):
        self._timeout_id = None
        self._gconf_key = gconf_key
        client = gconf.client_get_default()
        self._notify_id = client.notify_add(gconf_key, self.__gconf_notify_cb)
        initial_value = self._get_gconf_value()
        self._undo_value = initial_value
        self.set_value_from_gconf(initial_value)
        widget = widget or self
        widget.connect(signal, self.__changed_cb)

    def undo(self):
        """Revert to original value if modified"""
        if not self.changed:
            return

        logging.debug('Reverting %r to %r', self._gconf_key, self._undo_value)
        self._set_gconf_value(self._undo_value)

    def get_value_for_gconf(self):
        """
        Return the current value of the widget in a format suitable for GConf

        MUST be implemented by subclasses.
        """
        raise NotImplementedError()

    def set_value_from_gconf(self, value):
        """
        Set the current value of the widget based on a value from GConf

        MUST be implemented by subclasses.
        """
        raise NotImplementedError()

    def __changed_cb(self, widget):
        if self._timeout_id is not None:
            gobject.source_remove(self._timeout_id)

        self._timeout_id = gobject.timeout_add(_APPLY_TIMEOUT, self._commit,
                                               widget)

    def __gconf_notify_cb(self, client, transaction_id_, entry, user_data_):
        new_value = _gconf_value_to_python(entry.value)
        self.set_value_from_gconf(new_value)

    def _commit(self, widget):
        new_value = self.get_value_for_gconf()
        logging.debug('Setting %r to %r', self._gconf_key, new_value)

        widget.handler_block_by_func(self.__changed_cb)
        try:
            self._set_gconf_value(new_value)
        finally:
            widget.handler_unblock_by_func(self.__changed_cb)

    def _set_gconf_value(self, new_value):
        client = gconf.client_get_default()
        gconf_type = client.get(self._gconf_key).type
        if gconf_type == gconf.VALUE_STRING:
            client.set_string(self._gconf_key, new_value)
        elif gconf_type == gconf.VALUE_INT:
            client.set_int(self._gconf_key, new_value)
        elif gconf_type == gconf.VALUE_FLOAT:
            client.set_float(self._gconf_key, new_value)
        elif gconf_type == gconf.VALUE_BOOL:
            client.set_bool(self._gconf_key, new_value)
        elif gconf_type == gconf.VALUE_LIST:
            list_type = client.get(self._gconf_key).get_list_type()
            client.set_list(self._gconf_key, list_type, new_value)
        else:
            raise TypeError('Cannot store %r in GConf' % (new_value, ))

    def _get_gconf_value(self):
        client = gconf.client_get_default()
        return _gconf_value_to_python(client.get(self._gconf_key))

    @property
    def changed(self):
        return self._undo_value != self.get_value_for_gconf()


class GConfEntry(gtk.Entry, GConfMixin):
    """Text entry backed by GConf

    It is the callers responsibility to call GConfClient.add_dir() for the
    GConf directory containing the key.
    """

    def __init__(self, gconf_key):
        gtk.Entry.__init__(self)
        GConfMixin.__init__(self, gconf_key)

    def get_value_for_gconf(self):
        return self.props.text

    def set_value_from_gconf(self, value):
        self.props.text = value


class GConfIntegerSpinButton(gtk.SpinButton, GConfMixin):
    """Integer SpinButton backed by GConf

    It is the callers responsibility to call GConfClient.add_dir() for the
    GConf directory containing the key.
    """

    def __init__(self, gconf_key, adjustment, climb_rate=0):
        gtk.SpinButton.__init__(self, adjustment, climb_rate=climb_rate)
        GConfMixin.__init__(self, gconf_key)

    def get_value_for_gconf(self):
        return self.get_value_as_int()

    def set_value_from_gconf(self, value):
        self.set_value(value)


class GConfStringListEntry(GConfEntry):
    """Text entry backed by a GConf list of strings"""

    def __init__(self, gconf_key, separator=','):
        self._separator = separator
        GConfEntry.__init__(self, gconf_key)

    def get_value_for_gconf(self):
        entries = self.props.text.split(self._separator)
        return [entry for entry in entries if entry]

    def set_value_from_gconf(self, value):
        self.props.text = self._separator.join(value)


class SettingBox(gtk.HBox):
    """
    Base class for "lines" on the screen representing configuration settings
    """

    def __init__(self, name, size_group=None):
        gtk.HBox.__init__(self, spacing=style.DEFAULT_SPACING)
        self.label = gtk.Label(name)
        self.label.modify_fg(gtk.STATE_NORMAL,
                             style.COLOR_SELECTION_GREY.get_gdk_color())
        self.label.set_alignment(1, 0.5)
        self.label.show()
        self.pack_start(self.label, expand=False)

        if size_group is not None:
            size_group.add_widget(self.label)


class GConfStringSettingBox(SettingBox):
    """A configuration line for a GConf string setting"""

    def __init__(self, name, gconf_key, size_group=None):
        SettingBox.__init__(self, name, size_group=size_group)
        self.string_entry = GConfEntry(gconf_key)
        self.string_entry.show()
        self.pack_start(self.string_entry, expand=True)

    def undo(self):
        """Revert to original value if modified"""
        self.string_entry.undo()

    @property
    def changed(self):
        return self.string_entry.changed


class GConfHostListSettingBox(GConfStringSettingBox):
    """A configuration line for a host list GConf setting"""

    def __init__(self, name, gconf_key, size_group=None):
        SettingBox.__init__(self, name, size_group=size_group)
        self.hosts_entry = GConfStringListEntry(gconf_key)
        self.hosts_entry.show()
        self.pack_start(self.hosts_entry, expand=True)

    def undo(self):
        """Revert to original value if modified"""
        self.hosts_entry.undo()

    @property
    def changed(self):
        return self.hosts_entry.changed


class GConfHostPortSettingBox(SettingBox):
    """A configuration line for a combined host name and port GConf setting"""

    def __init__(self, name, host_key, port_key, size_group=None):
        SettingBox.__init__(self, name, size_group=size_group)
        self.host_name_entry = GConfEntry(host_key)
        self.host_name_entry.show()
        self.pack_start(self.host_name_entry, expand=True)

        # port number 0 means n/a
        adjustment = gtk.Adjustment(0, 0, 65535, 1, 10)
        self.port_spin_button = GConfIntegerSpinButton(port_key, adjustment,
                                                       climb_rate=0.1)
        self.port_spin_button.show()
        self.pack_start(self.port_spin_button, expand=False)

    def undo(self):
        """Revert to original values if modified"""
        self.host_name_entry.undo()
        self.port_spin_button.undo()

    @property
    def changed(self):
        return self.host_name_entry.changed or self.port_spin_button.changed


class ExclusiveOptionSetsBox(gtk.VBox):
    """
    Container for sets of different settings selected by a top-level setting

    Renders the top level setting as a ComboBox. Only the currently
    active set is shown on screen.
    """

    def __init__(self, top_name, option_sets, size_group=None):
        """Initialize an ExclusiveOptionSetsBox instance

        Arguments:

        top_name -- text label used for the top-level selection
        option_sets -- list of tuples containing text label and GTK
                       widget to display for each of the option sets
        size_group -- optional gtk.SizeGroup to use for the top-level label
        """
        gtk.VBox.__init__(self, spacing=style.DEFAULT_SPACING)
        self.label_size_group = size_group
        top_box = gtk.HBox(spacing=style.DEFAULT_SPACING)
        top_box.show()
        top_label = gtk.Label(top_name)
        top_label.modify_fg(gtk.STATE_NORMAL,
                            style.COLOR_SELECTION_GREY.get_gdk_color())
        top_label.set_alignment(1, 0.5)
        top_label.show()
        self.label_size_group.add_widget(top_label)
        top_box.pack_start(top_label, expand=False)

        model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_OBJECT)
        self._top_combo_box = gtk.ComboBox(model=model)
        self._top_combo_box.connect('changed', self.__combo_changed_cb)
        self._top_combo_box.show()

        cell_renderer = gtk.CellRendererText()
        cell_renderer.props.ellipsize = pango.ELLIPSIZE_MIDDLE
        cell_renderer.props.ellipsize_set = True
        self._top_combo_box.pack_start(cell_renderer)
        self._top_combo_box.add_attribute(cell_renderer, 'text', 0)
        top_box.pack_start(self._top_combo_box, expand=True)
        self.pack_start(top_box, expand=False)

        self._settings_box = gtk.VBox()
        self._settings_box.show()
        self.pack_start(self._settings_box, expand=False)

        for name, box in option_sets:
            model.append((name, box))

    def __combo_changed_cb(self, combobox):
        giter = combobox.get_active_iter()
        new_box = combobox.get_model().get(giter, 1)[0]
        current_box = self._settings_box.get_children()
        if current_box:
            self._settings_box.remove(current_box[0])

        self._settings_box.add(new_box)
        new_box.show()


class GConfExclusiveOptionSetsBox(ExclusiveOptionSetsBox, GConfMixin):
    """
    Container for sets of GConf settings based on a top-level setting
    """

    def __init__(self, top_name, top_gconf_key, option_sets, size_group=None):
        """Initialize a GConfExclusiveOptionSetsBox instance

        Arguments:

        top_name -- text label used for the top-level selection
        top_gconf_key -- key for the GConf entry to use for the
                         top-level selection
        option_sets -- list of tuples containing text label, matching
                       GConf value as well as the GTK widget to display
                       for each of the option sets
        size_group -- optional gtk.SizeGroup to use for the top-level label
        """
        display_sets = [(name, widget) for name, value, widget in option_sets]
        self._top_mapping = dict([(name, value)
                                  for name, value, widget in option_sets])
        ExclusiveOptionSetsBox.__init__(self, top_name, display_sets,
                                        size_group=size_group)
        GConfMixin.__init__(self, top_gconf_key, self._top_combo_box)

    def get_value_for_gconf(self):
        giter = self._top_combo_box.get_active_iter()
        if giter is None:
            return None

        name = self._top_combo_box.get_model().get(giter, 0)[0]
        return self._top_mapping[name]

    def set_value_from_gconf(self, value):
        for idx, (name, widget_) in enumerate(self._top_combo_box.get_model()):
            if self._top_mapping[name] == value:
                self._top_combo_box.set_active(idx)
                return

        raise ValueError('Invalid value %r' % (value, ))


class OptionalSettingsBox(gtk.VBox):
    """
    Container for settings (de)activated by a top-level setting

    Renders the top level setting as a CheckButton. The settings are only
    shown on screen if the top-level setting is enabled.
    """

    def __init__(self, top_name, options):
        """Initialize an OptionalSettingsBox instance

        Arguments:

        top_name -- text label used for the top-level selection
        options -- list of GTK widgets to display for each of the options
        """
        gtk.VBox.__init__(self, spacing=style.DEFAULT_SPACING)
        self._top_check_button = gtk.CheckButton()
        self._top_check_button.props.label = top_name
        self._top_check_button.connect('toggled', self.__button_changed_cb)
        self._top_check_button.show()
        self.pack_start(self._top_check_button, expand=True)

        self._settings_box = gtk.VBox(spacing=style.DEFAULT_SPACING)
        self.pack_start(self._settings_box, expand=False)

        for box in options:
            self._settings_box.pack_start(box)

    def __button_changed_cb(self, check_button):
        if check_button.get_active():
            self._settings_box.show()
        else:
            self._settings_box.hide()


class GConfOptionalSettingsBox(OptionalSettingsBox, GConfMixin):
    """
    Container for GConf settings (de)activated by a top-level setting
    """

    def __init__(self, top_name, top_gconf_key, options):
        """Initialize a GConfExclusiveOptionSetsBox instance

        Arguments:

        top_name -- text label used for the top-level selection
        top_gconf_key -- key for the GConf entry to use for the
                         top-level selection
        options -- list of  GTK widgets to display for each of the options
        """
        OptionalSettingsBox.__init__(self, top_name, options)
        GConfMixin.__init__(self, top_gconf_key, self._top_check_button,
                            signal='toggled')

    def get_value_for_gconf(self):
        return self._top_check_button.get_active()

    def set_value_from_gconf(self, value):
        self._top_check_button.set_active(value)


class AddRemoveWidget(gtk.HBox):

    def __init__(self, label, add_button_clicked_cb,
                 remove_button_clicked_cb, index):
        gtk.Box.__init__(self)
        self.set_homogeneous(False)
        self.set_spacing(10)

        self._index = index
        self._add_button_added = False
        self._remove_button_added = False

        self._entry_box = gtk.Entry()
        self._entry_box.set_text(label)
        self.pack_start(self._entry_box, expand=False)
        self._entry_box.show()

        add_icon = Icon(icon_name='list-add')
        self._add_button = gtk.Button()
        self._add_button.set_image(add_icon)
        self._add_button.connect('clicked',
                                 add_button_clicked_cb,
                                 self)

        remove_icon = Icon(icon_name='list-remove')
        self._remove_button = gtk.Button()
        self._remove_button.set_image(remove_icon)
        self._remove_button.connect('clicked',
                                    remove_button_clicked_cb,
                                    self)

        self.__add_add_button()
        self.__add_remove_button()

    def _get_index(self):
        return self._index

    def _set_index(self, value):
        self._index = value

    def _get_entry(self):
        return self._entry_box.get_text()

    def __add_add_button(self):
        self.pack_start(self._add_button, expand=False)
        self._add_button.show()
        self._add_button_added = True

    def _remove_remove_button_if_not_already(self):
        if self._remove_button_added:
            self.__remove_remove_button()

    def __remove_remove_button(self):
        self.remove(self._remove_button)
        self._remove_button_added = False

    def _add_remove_button_if_not_already(self):
        if not self._remove_button_added:
            self.__add_remove_button()

    def __add_remove_button(self):
        self.pack_start(self._remove_button, expand=False)
        self._remove_button.show()
        self._remove_button_added = True


class MultiWidget(gtk.VBox):

    def __init__(self):
        gtk.VBox.__init__(self)

    def _add_widget(self, label):
        new_widget = AddRemoveWidget(label,
                                     self.__add_button_clicked_cb,
                                     self.__remove_button_clicked_cb,
                                     len(self.get_children()))
        self.add(new_widget)
        new_widget.show()
        self.show()
        self._update_remove_button_statuses()

    def __add_button_clicked_cb(self, add_button,
                                      add_button_container):
        self._add_widget('')
        self._update_remove_button_statuses()

    def __remove_button_clicked_cb(self, remove_button,
                                   remove_button_container):
        for child in self.get_children():
            if child._get_index() > remove_button_container._get_index():
                child._set_index(child._get_index() - 1)

        self.remove(remove_button_container)
        self._update_remove_button_statuses()

    def _update_remove_button_statuses(self):
        children = self.get_children()

        # Now, if there is only one entry, remove-button
        # should not be shown.
        if len(children) == 1:
            children[0]._remove_remove_button_if_not_already()

        # Alternatively, if there are more than 1 entries,
        # remove-button should be shown for all.
        if len(children) > 1:
            for child in children:
                child._add_remove_button_if_not_already()


    def _get_entries(self):
        entries = []
        for child in self.get_children():
            entries.append(child._get_entry())

        return entries


class Network(SectionView):
    def __init__(self, model, alerts):
        SectionView.__init__(self)

        self._model = model
        self.restart_alerts = alerts
        self._jabber_sid = 0
        self._jabber_valid = True
        self._radio_valid = True
        self._jabber_change_handler = None
        self._radio_change_handler = None
        self._network_configuration_reset_handler = None
        self._undo_objects = []

        client = gconf.client_get_default()
        client.add_dir('/system/http_proxy', gconf.CLIENT_PRELOAD_ONELEVEL)
        client.add_dir('/system/proxy', gconf.CLIENT_PRELOAD_ONELEVEL)

        self.set_border_width(style.DEFAULT_SPACING * 2)
        self.set_spacing(style.DEFAULT_SPACING)
        group = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)

        self._radio_alert_box = gtk.HBox(spacing=style.DEFAULT_SPACING)
        self._jabber_alert_box = gtk.HBox(spacing=style.DEFAULT_SPACING)
        self._hidden_network_alert_box = gtk.HBox(spacing=style.DEFAULT_SPACING)
        self._nm_connection_editor_alert_box = gtk.HBox(spacing=style.DEFAULT_SPACING)

        workspace = gtk.VBox()
        workspace.show()

        separator_wireless = gtk.HSeparator()
        workspace.pack_start(separator_wireless, expand=False)
        separator_wireless.show()

        label_wireless = gtk.Label(_('Wireless'))
        label_wireless.set_alignment(0, 0)
        workspace.pack_start(label_wireless, expand=False)
        label_wireless.show()
        box_wireless = gtk.VBox()
        box_wireless.set_border_width(style.DEFAULT_SPACING * 2)
        box_wireless.set_spacing(style.DEFAULT_SPACING)

        radio_info = WrappedLabel(_('Turn off the wireless radio to save battery'
                                 ' life'))
        radio_info.set_alignment(0, 0)
        radio_info.show()
        box_wireless.pack_start(radio_info, expand=False)

        box_radio = gtk.HBox(spacing=style.DEFAULT_SPACING)
        self._button = gtk.CheckButton()
        self._button.set_alignment(0, 0)
        box_radio.pack_start(self._button, expand=False)
        self._button.show()

        label_radio = gtk.Label(_('Radio'))
        label_radio.set_alignment(0, 0.5)
        box_radio.pack_start(label_radio, expand=False)
        label_radio.show()

        box_wireless.pack_start(box_radio, expand=False)
        box_radio.show()

        self._radio_alert = InlineAlert()
        self._radio_alert_box.pack_start(self._radio_alert, expand=False)
        box_radio.pack_end(self._radio_alert_box, expand=False)
        self._radio_alert_box.show()
        if 'radio' in self.restart_alerts:
            self._radio_alert.props.msg = self.restart_msg
            self._radio_alert.show()

        history_info = WrappedLabel(_('Discard network history if you have'
                                   ' trouble connecting to the network.'))
        history_info.set_alignment(0, 0)
        history_info.show()
        box_wireless.pack_start(history_info, expand=False)

        box_clear_history = gtk.HBox(spacing=style.DEFAULT_SPACING)
        self._clear_history_button = gtk.Button()
        self._clear_history_button.set_label(_('Discard network history'))
        box_clear_history.pack_start(self._clear_history_button, expand=False)
        if not self._model.have_networks():
            self._clear_history_button.set_sensitive(False)
        self._clear_history_button.show()
        box_wireless.pack_start(box_clear_history, expand=False)
        box_clear_history.show()

        workspace.pack_start(box_wireless, expand=False)
        box_wireless.show()

        separator_mesh = gtk.HSeparator()
        workspace.pack_start(separator_mesh, False)
        separator_mesh.show()

        label_mesh = gtk.Label(_('Collaboration'))
        label_mesh.set_alignment(0, 0)
        workspace.pack_start(label_mesh, expand=False)
        label_mesh.show()
        box_mesh = gtk.VBox()
        box_mesh.set_border_width(style.DEFAULT_SPACING * 2)
        box_mesh.set_spacing(style.DEFAULT_SPACING)

        server_info = WrappedLabel(_("The server is the equivalent of what"
                                  " room you are in; people on the same server"
                                  " will be able to see each other, even when"
                                  " they aren't on the same network."))
        server_info.set_alignment(0, 0)
        box_mesh.pack_start(server_info, expand=False)
        server_info.show()

        box_server = gtk.HBox(spacing=style.DEFAULT_SPACING)
        label_server = gtk.Label(_('Server:'))
        label_server.set_alignment(1, 0.5)
        label_server.modify_fg(gtk.STATE_NORMAL,
                               style.COLOR_SELECTION_GREY.get_gdk_color())
        box_server.pack_start(label_server, expand=False)
        group.add_widget(label_server)
        label_server.show()
        self._entry = gtk.Entry()
        self._entry.set_alignment(0)
        self._entry.modify_bg(gtk.STATE_INSENSITIVE,
                        style.COLOR_WHITE.get_gdk_color())
        self._entry.modify_base(gtk.STATE_INSENSITIVE,
                          style.COLOR_WHITE.get_gdk_color())
        self._entry.set_size_request(int(gtk.gdk.screen_width() / 3), -1)
        box_server.pack_start(self._entry, expand=False)
        self._entry.show()
        box_mesh.pack_start(box_server, expand=False)
        box_server.show()

        self._jabber_alert = InlineAlert()
        label_jabber_error = gtk.Label()
        group.add_widget(label_jabber_error)
        self._jabber_alert_box.pack_start(label_jabber_error, expand=False)
        label_jabber_error.show()
        self._jabber_alert_box.pack_start(self._jabber_alert, expand=False)
        box_mesh.pack_end(self._jabber_alert_box, expand=False)
        self._jabber_alert_box.show()
        if 'jabber' in self.restart_alerts:
            self._jabber_alert.props.msg = self.restart_msg
            self._jabber_alert.show()

        workspace.pack_start(box_mesh, expand=False)
        box_mesh.show()

        proxy_separator = gtk.HSeparator()
        workspace.pack_start(proxy_separator, False)
        proxy_separator.show()

        self._add_proxy_section(workspace)

        if self._model.is_hidden_network_connect_package_available():
            separator_hidden_network = gtk.HSeparator()
            workspace.pack_start(separator_hidden_network, False)
            separator_hidden_network.show()

            label_hidden_network = gtk.Label(_('Hidden Networks'))
            label_hidden_network.set_alignment(0, 0)
            workspace.pack_start(label_hidden_network, expand=False)
            label_hidden_network.show()
            box_hidden_network = gtk.VBox()
            box_hidden_network.set_border_width(style.DEFAULT_SPACING * 2)
            box_hidden_network.set_spacing(style.DEFAULT_SPACING)

            info = WrappedLabel(_("Enter the SSIDs of the hidden networks."))
            info.set_alignment(0, 0)
            box_hidden_network.pack_start(info, expand=False)
            info.show()

            self._hidden_network_alert = InlineAlert()
            self.hidden_network_msg = EXPLICIT_REBOOT_MESSAGE
            self._hidden_network_alert.props.msg =_(self.hidden_network_msg)
            self._hidden_network_alert_box.pack_start(self._hidden_network_alert,
                                                        expand=False)
            box_hidden_network.pack_end(self._hidden_network_alert_box,
                                                expand=False)
            self._hidden_network_alert_box.show()
            self._hidden_network_alert.show()

            self._widget_table = MultiWidget()
            box_hidden_network.pack_start(self._widget_table, expand=False)
            self._widget_table.show()

            workspace.pack_start(box_hidden_network, expand=False)
            box_hidden_network.show()


        separator_nm_connection_editor = gtk.HSeparator()
        workspace.pack_start(separator_nm_connection_editor, False)
        separator_nm_connection_editor.show()

        label_nm_connection_editor = gtk.Label(_('Advanced Network Settings'))
        label_nm_connection_editor.set_alignment(0, 0)
        workspace.pack_start(label_nm_connection_editor, expand=False)
        label_nm_connection_editor.show()

        box_nm_connection_editor = gtk.VBox()
        box_nm_connection_editor.set_border_width(style.DEFAULT_SPACING * 2)
        box_nm_connection_editor.set_spacing(style.DEFAULT_SPACING)

        info = WrappedLabel(_("For more specific network settings, use "
                              "the NetworkManager Connection Editor."))
        info.set_alignment(0, 0)
        box_nm_connection_editor.pack_start(info, expand=False)

        self._nm_connection_editor_alert = InlineAlert()
        self._nm_connection_editor_alert.props.msg = EXPLICIT_REBOOT_MESSAGE
        self._nm_connection_editor_alert_box.pack_start(self._nm_connection_editor_alert,
                expand=False)
        box_nm_connection_editor.pack_end(self._nm_connection_editor_alert_box,
                expand=False)
        self._nm_connection_editor_alert_box.show()
        self._nm_connection_editor_alert.show()

        launch_button = gtk.Button()
        launch_button.set_alignment(0, 0)
        launch_button.set_label(_('Launch'))
        launch_button.connect('clicked', self.__launch_button_clicked_cb)
        box_launch_button = gtk.HBox()
        box_launch_button.set_homogeneous(False)
        box_launch_button.pack_start(launch_button, expand=False)
        box_launch_button.show_all()

        box_nm_connection_editor.pack_start(box_launch_button, expand=False)
        workspace.pack_start(box_nm_connection_editor, expand=False)
        box_nm_connection_editor.show_all()


        scrolled = gtk.ScrolledWindow()
        scrolled.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        scrolled.add_with_viewport(workspace)
        scrolled.show()
        self.add(scrolled)

        self.setup()

    def _add_proxy_section(self, workspace):
        proxy_title = gtk.Label(_('Proxy'))
        proxy_title.set_alignment(0, 0)
        proxy_title.show()
        workspace.pack_start(proxy_title, expand=False)

        proxy_box = gtk.VBox()
        proxy_box.set_border_width(style.DEFAULT_SPACING * 2)
        proxy_box.set_spacing(style.DEFAULT_SPACING)
        proxy_box.show()

        proxy_info = WrappedLabel(_("Configure proxies to access the Internet."))
        proxy_info.set_alignment(0, 0)
        proxy_box.pack_start(proxy_info, expand=False)
        proxy_info.show()

        workspace.pack_start(proxy_box)

        size_group = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)

        automatic_proxy_box = gtk.VBox()
        automatic_proxy_box.set_spacing(style.DEFAULT_SPACING)

        url_box = GConfStringSettingBox(_('Configuration URL:'),
                                        '/system/proxy/autoconfig_url',
                                        size_group)
        url_box.show()
        automatic_proxy_box.pack_start(url_box)
        self._undo_objects.append(url_box)

        wpad_help_text = _('Web Proxy Autodiscovery (WPAD) is used when a'
                           ' Configuration URL is not provided. This is not'
                           ' recommended for untrusted public networks.')
        automatic_proxy_help = WrappedLabel(wpad_help_text)
        automatic_proxy_help.set_alignment(0, 0)
        automatic_proxy_help.show()
        automatic_proxy_box.pack_start(automatic_proxy_help)

        manual_proxy_box = gtk.VBox()
        manual_proxy_box.set_spacing(style.DEFAULT_SPACING)

        http_box = GConfHostPortSettingBox(_('HTTP Proxy:'),
                                           '/system/http_proxy/host',
                                           '/system/http_proxy/port',
                                           size_group)
        http_box.show()
        manual_proxy_box.pack_start(http_box)
        self._undo_objects.append(http_box)

        user_name_box = GConfStringSettingBox(_('Username:'),
            '/system/http_proxy/authentication_user', size_group)
        user_name_box.show()
        self._undo_objects.append(user_name_box)

        password_box = GConfStringSettingBox(_('Password:'),
            '/system/http_proxy/authentication_password', size_group)
        password_box.show()
        self._undo_objects.append(password_box)

        auth_box = GConfOptionalSettingsBox(_('Use authentication'),
            '/system/http_proxy/use_authentication',
            [user_name_box, password_box])
        auth_box.show()
        manual_proxy_box.pack_start(auth_box)
        self._undo_objects.append(auth_box)

        https_box = GConfHostPortSettingBox(_('HTTPS Proxy:'),
                                            '/system/proxy/secure_host',
                                            '/system/proxy/secure_port',
                                            size_group)
        https_box.show()
        manual_proxy_box.pack_start(https_box)
        self._undo_objects.append(https_box)

        ftp_box = GConfHostPortSettingBox(_('FTP Proxy:'),
                                          '/system/proxy/ftp_host',
                                          '/system/proxy/ftp_port',
                                          size_group)
        ftp_box.show()
        manual_proxy_box.pack_start(ftp_box)
        self._undo_objects.append(ftp_box)

        socks_box = GConfHostPortSettingBox(_('SOCKS Proxy:'),
                                            '/system/proxy/socks_host',
                                            '/system/proxy/socks_port',
                                            size_group)
        socks_box.show()
        manual_proxy_box.pack_start(socks_box)
        self._undo_objects.append(socks_box)

        option_sets = [('None', 'none', gtk.VBox()),
                       ('Automatic', 'auto', automatic_proxy_box),
                       ('Manual', 'manual', manual_proxy_box)]
        option_sets_box = GConfExclusiveOptionSetsBox(_('Method:'),
                                                      '/system/proxy/mode',
                                                      option_sets, size_group)
        option_sets_box.show()
        proxy_box.pack_start(option_sets_box, expand=False)
        self._undo_objects.append(option_sets_box)

        no_proxy_box = GConfHostListSettingBox(_('Ignored Hosts'),
            '/system/http_proxy/ignore_hosts', size_group)
        no_proxy_box.show()
        proxy_box.pack_start(no_proxy_box, expand=False)
        self._undo_objects.append(no_proxy_box)

    def setup(self):
        self._entry.set_text(self._model.get_jabber())
        try:
            radio_state = self._model.get_radio()
        except self._model.ReadError, detail:
            self._radio_alert.props.msg = detail
            self._radio_alert.show()
        else:
            self._button.set_active(radio_state)

        self._jabber_valid = True
        self._radio_valid = True
        self._radio_change_handler = self._button.connect( \
                'toggled', self.__radio_toggled_cb)
        self._jabber_change_handler = self._entry.connect( \
                'changed', self.__jabber_changed_cb)
        self._network_configuration_reset_handler =  \
                self._clear_history_button.connect( \
                        'clicked', self.__network_configuration_reset_cb)

        if self._model.is_hidden_network_connect_package_available():

            ssids = self._model.get_ssids()
            if len(ssids) == 0:
                self._widget_table._add_widget('')
            else:
                for ssid in ssids:
                    self._widget_table._add_widget(ssid)

    def undo(self):
        self._button.disconnect(self._radio_change_handler)
        self._entry.disconnect(self._jabber_change_handler)
        self._model.undo()
        self._jabber_alert.hide()
        self._radio_alert.hide()
        for setting in self._undo_objects:
            setting.undo()

    # pylint: disable=E0202
    @property
    def needs_restart(self):
        # Some parts of Sugar as well as many non-Gnome applications
        # use environment variables rather than gconf for proxy
        # settings, so we need to restart for the changes to take
        # _full_ effect.
        for setting in self._undo_objects:
            if setting.changed:
                return True

        return False

    # pylint: disable=E0102,E1101
    @needs_restart.setter
    def needs_restart(self, value):
        # needs_restart is a property (i.e. gets calculated) in this Control
        # Panel, but SectionView.__init__() wants to initialise it to False,
        # so we need to provide a (fake) setter.
        pass

    def _validate(self):
        if self._jabber_valid and self._radio_valid:
            self.props.is_valid = True
        else:
            self.props.is_valid = False

    def __radio_toggled_cb(self, widget, data=None):
        radio_state = widget.get_active()
        try:
            self._model.set_radio(radio_state)
        except self._model.ReadError, detail:
            self._radio_alert.props.msg = detail
            self._radio_valid = False
        else:
            self._radio_valid = True
            if self._model.have_networks():
                self._clear_history_button.set_sensitive(True)

        self._validate()
        return False

    def __jabber_changed_cb(self, widget, data=None):
        if self._jabber_sid:
            gobject.source_remove(self._jabber_sid)
        self._jabber_sid = gobject.timeout_add(_APPLY_TIMEOUT,
                                               self.__jabber_timeout_cb,
                                               widget)

    def __jabber_timeout_cb(self, widget):
        self._jabber_sid = 0
        if widget.get_text() == self._model.get_jabber:
            return
        try:
            self._model.set_jabber(widget.get_text())
        except self._model.ReadError, detail:
            self._jabber_alert.props.msg = detail
            self._jabber_valid = False
            self._jabber_alert.show()
            self.restart_alerts.append('jabber')
        else:
            self._jabber_valid = True
            self._jabber_alert.hide()

        for setting in self._undo_objects:
            setting.undo()

        self._validate()
        return False

    def __network_configuration_reset_cb(self, widget):
        # FIXME: takes effect immediately, not after CP is closed with
        # confirmation button
        self._model.clear_networks()
        if not self._model.have_networks():
            self._clear_history_button.set_sensitive(False)

    def perform_accept_actions(self):
        if self._model.is_hidden_network_connect_package_available():
            self._model.set_ssids(self._widget_table._get_entries())

    def __launch_button_clicked_cb(self, launch_button):
        self._model.launch_nm_connection_editor()


def _gconf_value_to_python(gconf_value):
    if gconf_value.type == gconf.VALUE_STRING:
        return gconf_value.get_string()
    elif gconf_value.type == gconf.VALUE_INT:
        return gconf_value.get_int()
    elif gconf_value.type == gconf.VALUE_FLOAT:
        return gconf_value.get_float()
    elif gconf_value.type == gconf.VALUE_BOOL:
        return gconf_value.get_bool()
    elif gconf_value.type == gconf.VALUE_LIST:
        return [_gconf_value_to_python(entry)
                for entry in gconf_value.get_list()]
    else:
        raise TypeError("Don't know how to handle GConf value"
                        " type %r" % (gconf_value.type, ))
